@@ -292,7 +292,8 @@ class DataSet(Sized):
                  specs: Optional[SpecsOrInterDeps] = None,
                  values: Optional[VALUES] = None,
                  metadata: Optional[Mapping[str, Any]] = None,
-                 shapes: Optional[Shapes] = None) -> None:
+                 shapes: Optional[Shapes] = None,
+                 in_memory_cache: bool = True) -> None:
         """
         Create a new :class:`.DataSet` object. The object can either hold a new run or
         an already existing run. If a ``run_id`` is provided, then an old run is
@@ -322,6 +323,9 @@ class DataSet(Sized):
                 of the data captured as a list of integers. The list is in the
                 same order as the interdependencies or paramspecs provided.
                 Ignored if ``run_id`` is provided.
+            in_memory_cache: Should measured data be keep in memory
+                and available as part of the `dataset.cache` object.
+
         """
         self.conn = conn_from_dbpath_or_conn(conn, path_to_db)
 
@@ -331,6 +335,7 @@ class DataSet(Sized):
         #: In memory representation of the data in the dataset.
         self.cache: DataSetCache = DataSetCache(self)
         self._results: List[Dict[str, VALUE]] = []
+        self._in_memory_cache = in_memory_cache
 
         if run_id is not None:
             if not run_exists(self.conn, run_id):
@@ -1055,11 +1060,7 @@ class DataSet(Sized):
             a column and a indexed by a :py:class:`pandas.MultiIndex` formed
             by the dependencies.
         """
-        datadict = self.get_parameter_data(*params,
-                                           start=start,
-                                           end=end)
-        dfs_dict = self._load_to_dataframe_dict(datadict)
-        return dfs_dict
+        return self.to_pandas_dataframe_dict(*params, start=start, end=end)
 
     def to_pandas_dataframe(self,
                             *params: Union[str,
@@ -1069,7 +1070,7 @@ class DataSet(Sized):
                             end: Optional[int] = None) -> "pd.DataFrame":
         """
         Returns the values stored in the :class:`.DataSet` for the specified parameters
-        and their dependencies as a concatinated :py:class:`pandas.DataFrame` s
+        and their dependencies as a concatenated :py:class:`pandas.DataFrame` s
 
         The DataFrame contains a column for the data and is indexed by a
         :py:class:`pandas.MultiIndex` formed from all the setpoints
@@ -1102,17 +1103,24 @@ class DataSet(Sized):
 
         Example:
             Return a pandas DataFrame with
-                df = ds.get_data_as_pandas_dataframe()
+                df = ds.to_pandas_dataframe()
         """
-        import pandas as pd
         datadict = self.get_parameter_data(*params,
                                            start=start,
                                            end=end)
+        return self._load_to_concatenated_dataframe(datadict)
+
+    def _load_to_concatenated_dataframe(self, datadict: ParameterData
+                                        ) -> "pd.DataFrame":
+        import pandas as pd
 
         if not self._same_setpoints(datadict):
             warnings.warn(
-                'Independent parameter setpoints are not equal. \
-                Check concatenated output carefully.')
+                "Independent parameter setpoints are not equal. "
+                "Check concatenated output carefully. Please "
+                "consider using `to_pandas_dataframe_dict` to export each "
+                "independent parameter to its own dataframe."
+            )
 
         dfs_dict = self._load_to_dataframe_dict(datadict)
         df = pd.concat(list(dfs_dict.values()), axis=1)
@@ -1162,7 +1170,10 @@ class DataSet(Sized):
 
                 dataarray_dict = ds.to_xarray_dataarray_dict()
         """
-        return self._load_to_xarray_dataarray_dict(*params, start=start, end=end)
+        data = self.get_parameter_data(*params,
+                                       start=start,
+                                       end=end)
+        return self._load_to_xarray_dataarray_dict(data)
 
     def to_xarray_dataset(self, *params: Union[str,
                                                ParamSpec,
@@ -1202,17 +1213,24 @@ class DataSet(Sized):
 
                 xds = ds.to_xarray_dataset()
         """
+        data = self.get_parameter_data(*params,
+                                       start=start,
+                                       end=end)
+
+        return self._load_to_xarray_dataset(data)
+
+    def _load_to_xarray_dataset(self, data: ParameterData) -> "xr.Dataset":
         import xarray as xr
 
-        if not self._same_setpoints(self.get_parameter_data(*params,
-                                                            start=start,
-                                                            end=end)):
+        if not self._same_setpoints(data):
             warnings.warn(
-                'Independent parameter setpoints are not equal. \
-                Check concatenated output carefully.')
+                "Independent parameter setpoints are not equal. "
+                "Check concatenated output carefully. Please "
+                "consider using `to_xarray_dataarray_dict` to export each "
+                "independent parameter to its own datarray."
+            )
 
-        data_xrdarray_dict = self._load_to_xarray_dataarray_dict(
-            *params, start=start, end=end)
+        data_xrdarray_dict = self._load_to_xarray_dataarray_dict(data)
 
         # Casting Hashable for the key type until python/mypy#1114
         # and python/typing#445 are resolved.
@@ -1275,16 +1293,9 @@ class DataSet(Sized):
         return dfs
 
     def _load_to_xarray_dataarray_dict(self,
-                                       *params: Union[str,
-                                                      ParamSpec,
-                                                      _BaseParameter],
-                                       start: Optional[int] = None,
-                                       end: Optional[int] = None) -> \
+                                       datadict: Dict[str, Dict[str, numpy.ndarray]]) -> \
             Dict[str, "xr.DataArray"]:
         import xarray as xr
-        datadict = self.get_parameter_data(*params,
-                                           start=start,
-                                           end=end)
 
         data_xrdarray_dict: Dict[str, xr.DataArray] = {}
 
@@ -1335,7 +1346,7 @@ class DataSet(Sized):
                                in a single file but no filename provided.
         """
         import pandas as pd
-        dfdict = self.get_data_as_pandas_dataframe()
+        dfdict = self.to_pandas_dataframe_dict()
         dfs_to_save = list()
         for parametername, df in dfdict.items():
             if not single_file:
@@ -1452,7 +1463,7 @@ class DataSet(Sized):
         Enqueue the results into self._results
 
         Before we can enqueue the results, all values of the results dict
-        must have the same length. We enqueue each parameter tree seperately,
+        must have the same length. We enqueue each parameter tree separately,
         effectively mimicking making one call to add_results per parameter
         tree.
 
@@ -1465,13 +1476,28 @@ class DataSet(Sized):
 
         toplevel_params = (set(interdeps.dependencies)
                            .intersection(set(result_dict)))
+        if self._in_memory_cache:
+            new_results: Dict[str, Dict[str, numpy.ndarray]] = {}
         for toplevel_param in toplevel_params:
             inff_params = set(interdeps.inferences.get(toplevel_param, ()))
             deps_params = set(interdeps.dependencies.get(toplevel_param, ()))
             all_params = (inff_params
                           .union(deps_params)
                           .union({toplevel_param}))
-            res_dict: Dict[str, VALUE] = {}  # the dict to append to _results
+
+            if self._in_memory_cache:
+                new_results[toplevel_param.name] = {}
+                new_results[toplevel_param.name][toplevel_param.name] = self._reshape_array_for_cache(
+                    toplevel_param,
+                    result_dict[toplevel_param]
+                )
+                for param in all_params:
+                    if param is not toplevel_param:
+                        new_results[toplevel_param.name][param.name] = self._reshape_array_for_cache(
+                            param,
+                            result_dict[param]
+                        )
+
             if toplevel_param.type == 'array':
                 res_list = self._finalize_res_dict_array(
                     result_dict, all_params)
@@ -1480,7 +1506,7 @@ class DataSet(Sized):
                     result_dict, toplevel_param,
                     inff_params, deps_params)
             else:
-                res_dict = {ps.name: result_dict[ps] for ps in all_params}
+                res_dict: Dict[str, VALUE] = {ps.name: result_dict[ps] for ps in all_params}
                 res_list = [res_dict]
             self._results += res_list
 
@@ -1492,6 +1518,36 @@ class DataSet(Sized):
         if standalones:
             stdln_dict = {st: result_dict[st] for st in standalones}
             self._results += self._finalize_res_dict_standalones(stdln_dict)
+            if self._in_memory_cache:
+                for st in standalones:
+                    new_results[st.name] = {
+                        st.name: self._reshape_array_for_cache(st, result_dict[st])
+                    }
+
+        if self._in_memory_cache:
+            self.cache.add_data(new_results)
+
+    @staticmethod
+    def _reshape_array_for_cache(
+            param: ParamSpecBase,
+            param_data: numpy.ndarray
+    ) -> numpy.ndarray:
+        """
+        Shape cache data so it matches data read from database.
+        This means:
+
+        - Add an extra singleton dim to array data
+        - flatten non array data into a linear array.
+        """
+        param_data = numpy.atleast_1d(param_data)
+        if param.type == "array":
+            new_data = numpy.reshape(
+                param_data,
+                (1,) + param_data.shape
+            )
+        else:
+            new_data = param_data.ravel()
+        return new_data
 
     @staticmethod
     def _finalize_res_dict_array(
@@ -1794,7 +1850,9 @@ def new_data_set(name: str,
                  specs: Optional[SPECS] = None,
                  values: Optional[VALUES] = None,
                  metadata: Optional[Any] = None,
-                 conn: Optional[ConnectionPlus] = None) -> DataSet:
+                 conn: Optional[ConnectionPlus] = None,
+                 in_memory_cache: bool = True,
+                 ) -> DataSet:
     """
     Create a new dataset in the currently active/selected database.
 
@@ -1807,6 +1865,8 @@ def new_data_set(name: str,
         specs: list of parameters to create this dataset with
         values: the values to associate with the parameters
         metadata: the metadata to associate with the dataset
+        in_memory_cache: Should measured data be keep in memory
+            and available as part of the `dataset.cache` object.
 
     Return:
         the newly created :class:`.DataSet`
@@ -1815,7 +1875,7 @@ def new_data_set(name: str,
     # in `Runner` to pass a connection from an existing `Experiment`.
     d = DataSet(path_to_db=None, run_id=None, conn=conn,
                 name=name, specs=specs, values=values,
-                metadata=metadata, exp_id=exp_id)
+                metadata=metadata, exp_id=exp_id, in_memory_cache=in_memory_cache)
 
     return d
 
