@@ -20,6 +20,7 @@ from qcodes.tests.instrument_mocks import (ArraySetPointParam,
                                            DummyChannelInstrument,
                                            DummyInstrument,
                                            Multi2DSetPointParam,
+                                           Multi2DSetPointParam2Sizes,
                                            setpoint_generator)
 from qcodes.utils.validators import Arrays, ComplexNumbers, Numbers
 
@@ -157,7 +158,7 @@ def scalar_dataset(dataset):
 
     dataset.set_interdependencies(idps)
     dataset.mark_started()
-    dataset.add_results([{p.name: np.int(n_rows*10*pn+i)
+    dataset.add_results([{p.name: int(n_rows*10*pn+i)
                           for pn, p in enumerate(all_params)}
                          for i in range(n_rows)])
     dataset.mark_completed()
@@ -199,6 +200,7 @@ def array_dataset(experiment, request):
     finally:
         datasaver.dataset.conn.close()
 
+
 @pytest.fixture(scope="function",
                 params=["array", "numeric"])
 def array_dataset_with_nulls(experiment, request):
@@ -247,6 +249,22 @@ def multi_dataset(experiment, request):
         datasaver.dataset.conn.close()
 
 
+@pytest.fixture(scope="function",
+                params=["array"])
+def different_setpoint_dataset(experiment, request):
+    meas = Measurement()
+    param = Multi2DSetPointParam2Sizes()
+
+    meas.register_parameter(param, paramtype=request.param)
+
+    with meas.run() as datasaver:
+        datasaver.add_result((param, param.get(),))
+    try:
+        yield datasaver.dataset
+    finally:
+        datasaver.dataset.conn.close()
+
+
 @pytest.fixture(scope="function")
 def array_in_scalar_dataset(experiment):
     meas = Measurement()
@@ -359,7 +377,7 @@ def standalone_parameters_dataset(dataset):
     dataset.set_interdependencies(idps)
 
     dataset.mark_started()
-    dataset.add_results([{p.name: np.int(n_rows*10*pn+i)
+    dataset.add_results([{p.name: int(n_rows*10*pn+i)
                           for pn, p in enumerate(params_all)}
                          for i in range(n_rows)])
     dataset.mark_completed()
@@ -431,7 +449,7 @@ def some_interdeps():
     ps5 = ParamSpecBase('ps5', paramtype='numeric', label='Signal',
                         unit='Conductance')
     ps6 = ParamSpecBase('ps6', paramtype='text', label='Goodness',
-                    unit='')
+                        unit='')
 
     idps = InterDependencies_(dependencies={ps5: (ps3, ps4), ps6: (ps3, ps4)},
                               inferences={ps4: (ps2,), ps3: (ps1,)})
@@ -545,14 +563,12 @@ def complex_num_instrument():
                             get_cmd=lambda: np.arange(5),
                             set_cmd=False)
 
-
     dummyinst.add_parameter('some_complex_array',
                             label='Some Array',
                             unit='some_array_unit',
                             get_cmd=lambda: np.ones(5) + 1j*np.ones(5),
                             set_cmd=False)
 
-
     yield dummyinst
     dummyinst.close()
 
